/**
            Hello, Creative maker.
            This code allows you to create your own DIY HID device that functions as an air mouse with media key controls.
            You can customize which media keys to assign by modifying the following lines:

               detectMediaKeyPress(touchStatus, lastTouchStatus, 3, KEY_MEDIA_REWIND, "REWIND");  
               detectMediaKeyPress(touchStatus, lastTouchStatus, 4, KEY_MEDIA_FASTFORWARD, "FASTFORWARD");  
               detectMediaKeyPress(touchStatus, lastTouchStatus, 5, KEY_MEDIA_VOLUMEUP, "VOLUMEUP");  
               detectMediaKeyPress(touchStatus, lastTouchStatus, 6, KEY_MEDIA_NEXTTRACK, "NEXTTRACK");  
               detectMediaKeyPress(touchStatus, lastTouchStatus, 7, KEY_MEDIA_PLAYPAUSE, "Play/Pause");  
               detectMediaKeyPress(touchStatus, lastTouchStatus, 8, KEY_MEDIA_PREVIOUSTRACK, "PREVIOUSTRACK");  
               detectMediaKeyPress(touchStatus, lastTouchStatus, 9, KEY_MEDIA_VOLUMEDOWN, "VOLUMEDOWN");  
               detectMediaKeyPress(touchStatus, lastTouchStatus, 10, KEY_MEDIA_MUTE, "Mute");  

            For example, if you want to change touch point 3 from "REWIND" to "REPEAT," modify this line:

               detectMediaKeyPress(touchStatus, lastTouchStatus, 3, KEY_MEDIA_REWIND, "REWIND");  

            to

              detectMediaKeyPress(touchStatus, lastTouchStatus, 3, KEY_MEDIA_REPEAT, "Repeat");  

                                        Available Media Keys: 
                            You can use any of the following media keys:

            Playback Controls: KEY_MEDIA_PLAY, KEY_MEDIA_PAUSE, KEY_MEDIA_RECORD, KEY_MEDIA_PLAYPAUSE, KEY_MEDIA_STOP, KEY_MEDIA_EJECT, KEY_MEDIA_RANDOMPLAY, KEY_MEDIA_REPEAT
            Track Navigation: KEY_MEDIA_NEXTTRACK, KEY_MEDIA_PREVIOUSTRACK, KEY_MEDIA_FASTFORWARD, KEY_MEDIA_REWIND
            Volume Controls: KEY_MEDIA_VOLUMEUP, KEY_MEDIA_VOLUMEDOWN, KEY_MEDIA_MUTE
            Other Functions: KEY_MEDIA_WWWHOME, KEY_MEDIA_MYCOMPUTER, KEY_MEDIA_CALCULATOR, KEY_MEDIA_WWWFAVORITES, KEY_MEDIA_WWWSEARCH, KEY_MEDIA_WWWSTOP, KEY_MEDIA_WWWBACK, 
            KEY_MEDIA_MEDIASELECT, KEY_MEDIA_MAIL
                            Feel free to customize the touch points to suit your preferences.

             If you enjoy this project, consider subscribing to my YouTube channel Misfit Maker.

                                        Happy building! :)

**/
#include <Wire.h>
#include <MPU6050.h>
#include <Adafruit_MPR121.h>
#include <BleCompositeHID.h>
#include <KeyboardDevice.h>
#include <MouseDevice.h>

#define LED_PIN D2 // Define LED pin

// Declare objects for MPU6050 and MPR121
MPU6050 mpu;
Adafruit_MPR121 mpr121 = Adafruit_MPR121();

// BLE HID objects
KeyboardDevice* keyboard;
MouseDevice* mouse;
BleCompositeHID compositeHID("Misfit_HID", "Mystfit", 100);

// Touch key states for debounce
uint16_t lastTouchStatus = 0;

// Smoothing filter variables
const int numReadings = 10;
float pitchReadings[numReadings];
float rollReadings[numReadings];
int readIndex = 0;
float pitchFiltered = 0;
float rollFiltered = 0;

// Motion threshold for inactivity detection
const int motionThreshold = 5;
int previousDx = 0, previousDy = 0;

unsigned long lastTouchTime = 0;
const unsigned long debounceDelay = 50; // Adjust debounce delay as needed

void setup() {
    Serial.begin(115200);
    Wire.begin();
    pinMode(LED_PIN, OUTPUT);
    digitalWrite(LED_PIN, HIGH); // Start with LED on (flashing mode)

    // Initialize BLE Composite HID
    KeyboardConfiguration config;
    config.setUseMediaKeys(true);
    keyboard = new KeyboardDevice(config);

    MouseConfiguration mouseConfig;
    mouseConfig.setAutoReport(false);
    mouse = new MouseDevice(mouseConfig);

    compositeHID.addDevice(keyboard);
    compositeHID.addDevice(mouse);
    compositeHID.begin();
    delay(3000); // Allow time for BLE to initialize

    // MPU6050 initialization
    mpu.initialize();
    if (!mpu.testConnection()) {
        while (1);
    }

    // MPR121 initialization
    if (!mpr121.begin(0x5A)) {
        while (1);
    }

    // Set touch thresholds
    for (uint8_t i = 0; i < 12; i++) {
        mpr121.setThresholds(12, 6);
    }
}

void loop() {
    if (!compositeHID.isConnected()) {
        
    // Blink LED when searching for connection
        digitalWrite(LED_PIN, HIGH);
        delay(250);
        digitalWrite(LED_PIN, LOW);
        delay(250);
        return;
    }
    
    digitalWrite(LED_PIN, LOW); // Turn off LED upon connection

    // Read accelerometer values
    int16_t ax, ay, az;
    mpu.getAcceleration(&ax, &ay, &az);
    float normAx = ax / 16384.0;
    float normAy = ay / 16384.0;

    // Apply moving average filter
    pitchReadings[readIndex] = normAx;
    rollReadings[readIndex] = normAy;
    readIndex = (readIndex + 1) % numReadings;

    float pitchSum = 0, rollSum = 0;
    for (int i = 0; i < numReadings; i++) {
        pitchSum += pitchReadings[i];
        rollSum += rollReadings[i];
    }
    pitchFiltered = pitchSum / numReadings;
    rollFiltered = rollSum / numReadings;

    // Map motion to mouse movement
    int dx = map(rollFiltered * 100, -100, 100, -25, 25);
    int dy = map(pitchFiltered * 100, -100, 100, -25, 25);
    uint16_t touchStatus = mpr121.touched();

    // Move the mouse continuously based on tilt
    if (touchStatus & (1 << 0)) {
        mouse->mouseMove(dx, dy);
        mouse->sendMouseReport();
    } else {
        mouse->mouseMove(0, 0);
        mouse->sendMouseReport();
    }

    // Handle touch input with debounce
    if (millis() - lastTouchTime > debounceDelay) {
        lastTouchTime = millis();

        // Mouse button control
        if (touchStatus & (1 << 1)) {
            mouse->mousePress(MOUSE_LOGICAL_LEFT_BUTTON);
            mouse->sendMouseReport();
        } else {
            mouse->mouseRelease(MOUSE_LOGICAL_LEFT_BUTTON);
            mouse->sendMouseReport();
        }
    
        if (touchStatus & (1 << 2)) {
            mouse->mousePress(MOUSE_LOGICAL_RIGHT_BUTTON);
            mouse->sendMouseReport();
        } else {
            mouse->mouseRelease(MOUSE_LOGICAL_RIGHT_BUTTON);
            mouse->sendMouseReport();
        }
    }

    // Handle media key inputs
    detectMediaKeyPress(touchStatus, lastTouchStatus, 3, KEY_MEDIA_REWIND, "REWIND");
    detectMediaKeyPress(touchStatus, lastTouchStatus, 4, KEY_MEDIA_FASTFORWARD, "FASTFORWARD");
    detectMediaKeyPress(touchStatus, lastTouchStatus, 5, KEY_MEDIA_VOLUMEUP, "VOLUMEUP");
    detectMediaKeyPress(touchStatus, lastTouchStatus, 6, KEY_MEDIA_NEXTTRACK, "NEXTTRACK");
    detectMediaKeyPress(touchStatus, lastTouchStatus, 7, KEY_MEDIA_PLAYPAUSE, "Play/Pause");
    detectMediaKeyPress(touchStatus, lastTouchStatus, 8, KEY_MEDIA_PREVIOUSTRACK, "PREVIOUSTRACK");
    detectMediaKeyPress(touchStatus, lastTouchStatus, 9, KEY_MEDIA_VOLUMEDOWN, "VOLUMEDOWN");
    detectMediaKeyPress(touchStatus, lastTouchStatus, 10, KEY_MEDIA_MUTE, "Mute");

    lastTouchStatus = touchStatus;

    delay(20);
}

void detectMediaKeyPress(uint16_t currentTouch, uint16_t lastTouch, int bit, uint32_t keyCode, const char* action) {
    bool isCurrentlyTouched = currentTouch & (1 << bit);
    bool wasPreviouslyTouched = lastTouch & (1 << bit);

    if (isCurrentlyTouched && !wasPreviouslyTouched) {
        keyboard->mediaKeyPress(keyCode);
    }
    if (!isCurrentlyTouched && wasPreviouslyTouched) {
        keyboard->mediaKeyRelease(keyCode);
    }
}
